<?php
/**
 * Provides an abstract representation of data which can be filtered, sorted and paged.
 * 
 * The underlying data can be anything from an SQL connection to a text file. In this way the details of the
 * data storage are hidden.
 * 
 * The datasource also accepts a presentation object which transforms the data into the required output format. This way
 * the data can be presented as XML, JSON or just a HTML Table by creating a new presentation object and telling the datasource that
 * this is the presentation that it should output.
 * 
 * Replaces DP_List_Dynamic in earlier dp3 implementations.
 * 
 * @package dotproject
 * @subpackage lib
 * @author mosen
 */
class DP_DataSource
{
	/**
	 * @var mixed $id Datasource identifier
	 */
	protected $id;
	
	/**
	 * @var Zend_Db_Table $tbl Table gateway to underlying SQL table.
	 */
	protected $tbl;
	
	/**
	 * @var Zend_Db_Rowset $result_cache Cache of returned rows.
	 */
	protected $result_cache;
	
	/**
	 * @var DP_Presentation $default_presentation The default presentation layer to use.
	 */
	protected $default_presentation;
	
	/**
	 * @var DP_Filter $filter The filtering criteria for this datasource
	 */
	protected $filter;
	
	/**
	 * DP_DataSource constructor.
	 */
	public function __construct($id)
	{
		$this->id = $id;
		$this->filter = new DP_Filter();
	}
	
	/**
	 * Make column definitions using the table metadata extracted by Zend_Db_Table.
	 * 
	 * @return DP_Datasource_Columns instance of columns object.
	 */
	public function makeColsFromMetadata()
	{
		return new DP_Datasource_Columns($this->tbl);
	}
	
	// TEMP TODO: use an array presentation to fetch rows as array.
	public function fetchAll()
	{
		return $this->tbl->fetchAll();
	}
	
	/**
	 * Set the default presentation object to use when generating output.
	 *
	 * @param DP_Presentation $pres
	 * @return null
	 */
	public function setDefaultPresentation(DP_Presentation $pres)
	{
		$this->default_presentation = $pres;
	}
	
	/**
	 * Get the default presentation for this datasource.
	 * 
	 * The default presentation is set by the datasource factory, and usually determined on a per-module basis.
	 * 
	 * @return DP_Presentation Instance of a DP_Presentation Object
	 */
	public function getDefaultPresentation()
	{
		return $this->default_presentation;
	}
	
	/**
	 * Get the internal filter object
	 *
	 * @return DP_Filter the filter
	 */
	public function getFilter()
	{
		return $this->filter;
	}
	
	/**
	 * Set the internal filter object
	 *
	 * @param DP_Filter $filter
	 */
	public function setFilter(DP_Filter $filter)
	{
		$this->filter = $filter;
	}
	
	protected function _processFilters(Zend_Db_Select &$select)
	{
		$fi = $this->filter->getIterator();
						
		while ($fi->currentItem() != null) {
			$f = $fi->currentItem();
			
			switch ($f['filter_type']) {
				case DP_Filter::VALUE_EQUAL:
					$select->where($f['filter_field'] . " = " . $f['field_value']);
					break;
			}
			
			$fi->next();
		}
	}
	
	/**
	 * Fetch all rows using the given presentation object.
	 * 
	 * @param DP_Presentation $pres Instance of a DP_Presentation object, or null if using the default.
	 * @return Zend_Db_Result if valid. Otherwise -1 if presentation error is not valid.
	 */
	public function fetch($pres = null)
	{
		if ($pres == null) {	
			if ($pres == null && $this->default_presentation != null) {
				// Fetch using default layer
				$fields = $this->default_presentation->getDefaultFields();
				
				$select = $this->tbl->select();
				$select->from($this->tbl, $fields);
				

				$this->_processFilters($select);
				$rows = $this->tbl->fetchAll($select);
				
				return $this->default_presentation->translate($rows);
			} else {
				// No layer available.
				return -1;
			}		
		} else {
			// Fech using specified layer
			return -1;
		}
	}
	
	public function fetchRelated($relation_id, $pres = null) {
		if ($pres == null) {	
			if ($pres == null && $this->default_presentation != null) {
				// Fetch using default layer
				$fields = $this->default_presentation->getDefaultFields();
			
				
				$select = $this->tbl->select();
				$select->from('ous');
				$select->joinLeft('memberships', 'ous.id = memberships.child_id', array());
				if ($relation_id > 0) {
					$select->where('memberships.parent_id = '.$relation_id);
				} else {
					$select->where('memberships.parent_id IS NULL');
				}
				
				$writer = new Zend_Log_Writer_Firebug();
				$logger = new Zend_Log($writer);
		
				$logger->log( $select->__toString(), Zend_Log::INFO );
				$this->_processFilters($select);
				$rows = $this->tbl->fetchAll($select);
				
				return $this->default_presentation->translate($rows);
			} else {
				// No layer available.
				return -1;
			}		
		} else {
			// Fech using specified layer
			return -1;
		}
	}
	
	// Static Methods
	
	/**
	 * Retrieve an instance of a DP_DataSource by its module-local identifier
	 * 
	 * @param mixed $id Identifier of the requested datasource.
	 * @return DP_DataSource Instance of a DP_DataSource object.
	 */
	public static function factory($id)
	{
		return new DP_DataSource($id);	
	}
}
?>