<?php
/**
 * Stores column metadata and definitions.
 * 
 * The class can extract metadata from a Db_Table class and present the information as a javascript object for use
 * with a DataTable widget. This provides some sane defaults as to what the column headings, size defaults, validators,
 * and editors should be. The API should also provide methods to show/hide columns and change defaults.
 * 
 * This object is paired with a view helper which does the job of outputting the javascript code.
 * 
 * @author ebrosnan
 * @version 3.0 alpha
 * @package dotproject
 */
class DP_Datasource_Columns implements Iterator, Countable {
	/**
	 * @var array $table_meta Table column metadata.
	 */
	protected $table_meta; 
	/**
	 * @var array $table_meta_keys Array of keys to table column metadata.
	 */
	protected $table_meta_keys;
	/**
	 * @var integer $object_iter_idx Iterator index.
	 */
	protected $object_iter_idx;
	/**
	 * @var array $labels Column header labels.
	 */
	protected $labels;
	/**
	 * @var array $enabled_columns Column keys of columns that should be outputted.
	 */
	protected $enabled_columns;
	
	/**
	 * Datasource columns constructor.
	 * 
	 * @param Zend_Db_Table $tbl The instance of zend_db_table to extract column metadata from.
	 * @param Array $columns Optional array of column keys to use when generating from database.
	 */
	public function __construct(Zend_Db_Table $tbl, $columns = Array()) {
		$this->table_meta = $tbl->info('metadata');
		$this->table_meta_keys = array_keys($this->table_meta);
		$this->object_iter_idx = 0;
		if (count($columns) < 1) {
			$this->enabled_columns = $this->table_meta_keys;
		} else {
			$this->enabled_columns = $columns;
		}
		
	}
	
	/**
	 * Generate reasonable YUI column defaults for formatting and editing.
	 * 
	 * @param mixed $key Array key of a column in the table metadata.
	 * @param Array $column table column metadata
	 * 
	 * @return Array associative array representing a YUI DataTable column definition.
	 */
	private function _toYuiDef($key, array $column) {
		switch($column['DATA_TYPE'])
		{
			case 'int':
				$formatter = 'number';
				$editor = 'YAHOO.widget.DataTable.TextboxCellEditor';
				break;
			case 'varchar':
				$formatter = 'text';
				$editor = 'YAHOO.widget.DataTable.TextboxCellEditor';
				break;
			case 'text':
				$formatter = 'text';
				$editor = 'YAHOO.widget.DataTable.TextboxCellEditor';
				break;
			default:
				$formatter = 'text';
				$editor = 'YAHOO.widget.DataTable.TextboxCellEditor';
		}
		
		// Use the column name if the label is not available.
		$label = (isset($this->labels[$key])) ? $this->labels[$key] : $key;
		
		return array(
			"key"=>$key,
			"label"=>$label,
			"formatter"=>$formatter,
			"editor"=>$editor,
		
		);
	}
	
	public function setEnabled(Array $column_keys) 
	{
		$this->enabled_columns = $column_keys;
	}
	
	public function setLabels(Array $cols_labels)
	{
		$this->labels = $cols_labels;
	}
	
	// From Countable
	
	public function count() {
		return count($this->enabled_columns);
	}
	
	// From Iterator
	
	/**
	 * Return the current element.
	 */
	public function current() {
		if (count($this->enabled_columns) > count($this->table_meta)) {
			throw new Exception("Enabled more columns than are available in DP_Datasource_Columns");	
		}
		
		if ($meta = $this->table_meta[$this->enabled_columns[$this->object_iter_idx]]) {
			$meta_key = $this->enabled_columns[$this->object_iter_idx];
			$definition = $this->_toYuiDef($meta_key, $meta);
			
			return $definition;
		} else {
			throw new Exception("Enabled one or more columns that were not part of the selected table in DP_Datasource_Columns");
		}
	}
	
	/**
	 * Return the key of the current element.
	 */
	public function key() {
		if ($this->needs_refresh == false) {
			return $this->object_iter_idx;
		} else {
			return null;
		}
	}
	
	/**
	 * Move forward to next element.
	 */
	public function next() {
		$this->object_iter_idx++;
	}
	
	/**
	 * Rewind the Iterator to the first element.
	 */
	public function rewind() {
		$this->object_iter_idx = 0;
	}
	
	/**
	 * Check if there is a current element after calls to rewind() or next().
	 */
	public function valid() {
		if ($this->object_iter_idx < count($this->enabled_columns)) {
			return true;
		} else {
			return false;
		}
	}	
	
}
?>