<?php

/**
 * DP_List
 * 
 * Based on the old DP_List_Dynamic, This class provides a way of abstracting
 * a sortable, searchable list away from the specifics of SQL.
 * 
 * The developer provides a base query which returns all results, and then adds modifier objects
 * via the appendModifier() or insertModifier() methods. Most of these modifier objects are generated by
 * DP_View objects, which represent the state of a view widget.
 * 
 * Just before the view needs to render, the preRender() method is called.
 * 
 * This object then iterates through the modifiers in its local collection.. applying them to the base query.
 * In this way we can plug-in functionality such as sorting, searching, paging etc as necessary, without having to
 * rewrite these commonly used pieces of functionality.
 * 
 * Another benefit to this method is that all of the view widget state is guaranteed to match the conditions imposed upon the query.
 * 
 * @package dotproject
 * @subpackage DP
 */
class DP_List implements Countable, Iterator, ArrayAccess, SplObserver // implements Countable, Iterator, ArrayAccess, SplObserver, DP_View_List_DataSource
{
	// This is a redesign of DP_List_Dynamic, The problems with the first were the tight coupling of presentation and state.
	
	/**
	 * @var $tbl Zend_Db_Table Table gateway to use for main table.
	 */
	protected $tbl;
	
	/**
	 * @var $base_query Zend_Db_Select The base query to execute, before conditions have been added.
	 */
	protected $base_query;
	
	/**
	 * @var $modpagereset Boolean Whether the page number will reset if a modifier changes.
	 */
	protected $page_will_reset;
	
	/**
	 * @var $iterator_index Integer Index of the iterator, part of the Iterator interface implementation.
	 */
	protected $iterator_index;
	
	/**
	 * Array of modifier objects (which inherit DP_Modifier)
	 * 
	 * These can be filters (WHERE) sorters (ORDER) or pagers (LIMIT). Keys point to arrays of modifiers.
	 * The keys are sorted in numerically ascending order to determine how the modifiers are applied.
	 * 
	 * @var Array Collection of modifiers.
	 */
	protected $modifiers;
	
	/**
	 * Results of the last query.
	 * 
	 * Like DP_List_Dynamic::$object_list
	 * 
	 * @var Array Most recent query results.
	 */
	public $results;
	
	/**
	 * @var $needs_refresh Boolean Whether the result set needs regenerating or not, depending on whether the conditions have changed.
	 */
	protected $needs_refresh;
	
	/**
	 * Constructor for DP_List
	 * 
	 * @constructor
	 */
	public function __construct()
	{
		$this->results = Array();
		$this->iterator_index = 0;
		$this->needs_refresh = true;
		$this->modifiers = Array();
		$this->page_will_reset = true;
		/*
		 * DP_List_Dynamic had a column name to column descriptive title attribute. This should be in the view component not in the list.
		 */
	}
	
	/**
	 * Append a modifier on to the stack of modifiers.
	 * 
	 * The modifier will be appended to the end of the stack of modifiers.
	 * 
	 * @see DP_Modifier
	 * @param DP_Modifier $m Modifier to insert
	 */
	public function appendModifier($m)
	{
		$this->modifiers[] = $m;
		$m->attach($this); // Observe for changes.

		// Add to local collection based on class - At the end of the queue
		// Observe so that we are updated when changes are made.
		// Observation may be made on widget state class or just the rule itself.. havent decided :)
	}
	
	/**
	 * Insert a modifier into the stack of modifiers.
	 * 
	 * The modifier will by default be inserted at the top of the priority list.
	 * 
	 * @param DP_Modifier $m
	 * @param Integer $priority The priority, used to determine the order in which modifiers are executed.
	 */
	public function insertModifier(DP_Modifier $m, $priority = 0)
	{
		// Add to local collection based on class - At the start of the queue
		// Observe so that we are updated when changes are made.		
	}
	
	/**
	 * Set whether the list current page will reset if a modifier changes.
	 * 
	 * @param Boolean $reset [defaults to true] reset to page 1 if true.
	 * @return null
	 */
	public function setPageResetsWhenModifierChanges( $reset = true )
	{
		$this->page_will_reset = (bool)$reset;
	}
	
	/**
	 * Getter method to determine whether the page will reset if a modifier changes.
	 * 
	 * @return Boolean true if current page will reset.
	 */
	public function getPageResetsWhenModifierChanges()
	{
		return $this->page_will_reset;
	}
	
	/**
	 * Handle called when controller is about to render.
	 * 
	 * Provides a hook where we can perform or generate a query because no more state changes will occur.
	 */
	public function preRender()
	{
		foreach ($this->modifiers as $m) {
			$m->apply($this->base_query);
		}
		
		$this->refresh();
	}
	
	/**
	 * Refresh the internal result set by re-running the query.
	 * 
	 * If any modifiers have changed, this needs to be called in order to get current results.
	 * Because it would be too time consuming to call a refresh on every change. This is invoked both manually
	 * and automatically just before the view render.
	 * 
	 */
	public function refresh()
	{
		$this->results = $this->tbl->fetchAll($this->base_query);
		$this->needs_refresh = false;
		
		// Notify Pager Modifiers that the total number of items returned has changed. DP_Modifier_Pagers should probably
		// Observe a list refresh notification.
	}
	
	// Implemented From Iterator
		
	/**
	 * Return the current element.
	 */
	public function current() {
		if ($this->needs_refresh == false) {
			return $this->results[$this->iterator_index];
		}
	}
	
	/**
	 * Return the key of the current element.
	 */
	public function key() {
		if ($this->needs_refresh == false) {
			return $this->iterator_index;
		}
	}
	
	/**
	 * Move forward to next element.
	 */
	public function next() {
		$this->iterator_index++;
	}
	
	/**
	 * Rewind the Iterator to the first element.
	 */
	public function rewind() {
		$this->iterator_index = 0;
	}
	
	/**
	 * Check if there is a current element after calls to rewind() or next().
	 */
	public function valid() {
		if ($this->needs_refresh == false) {
			if ($this->iterator_index < count($this->results) && $this->iterator_index > -1) {
				return true;
			} else {
				return false;
			}
		}
	}	
	
	// Implemented From ArrayAccess
	
	public function offsetExists($offset) {
		if ($this->needs_refresh == false && array_key_exists($offset, $this->results)) {
			return true;
		} else {
			return false;
		}
	}
	
	public function offsetGet($offset) {
		if ($this->needs_refresh == false) {
			return $this->results[$offset];
		} else {
			return false;
		}
	}
	
	public function offsetSet($offset, $value) {
		// we dont implement setting, read only values.
		return false;
	}
	
	public function offsetUnset($offset) {
		// we dont implement unset method either.
		return false;
	}
	
	// Implemented From Countable
	
	/**
	 * Get the total count of rows, unpaged.
	 * 
	 * @return Integer number of rows.
	 */
	public function count()
	{
		if ($this->needs_refresh == false) {
			return count($this->results);
		} else {
			return 0;
		}
	}
	
	// Implemented From SplObserver
	
	/**
	 * Called when a SplSubjects state has changed.
	 * 
	 * @param Object $subject The object that has changed.
	 */
	public function update(SplSubject $subject) {

		$this->needs_refresh = true; // Subject change requires data refresh.
		// If the list filters have changed, reset page to 1.
	}
	
}