<?php

/**
 * DP Modifier filter class.
 * 
 * Contains one or more rules for filtering a result set. Usually used to hold the state
 * of a search widget.
 * 
 * @author ebrosnan
 * @package dotproject
 * @subpackage DP_Modifier
 */
class DP_Modifier_Filter extends DP_Modifier implements Iterator, SplSubject, DP_Originator_Interface
{
	// Operation types.
	const FILTER_EQUAL = 101;
	const FILTER_SUBSTRING = 102;
	const FILTER_NOTEQUAL = 103;
	
	// Operations to SQL
	public $ops_sql = Array(
		101 => "= ?",
		102 => "LIKE '%?%'",
		103 => "<> ?" 
	);
	
	/**
	 * @var Array of observers, implemented as part of SplSubject.
	 */
	protected $_observers;
	
	/**
	 * Array of filter rules.
	 * 
	 * The array is an associative array of field_name => sorting rule. Where the sorting rule is one of the class
	 * constants:
	 * DP_Modifier_Sort::SORT_ASCENDING or DP_Modifier_Sort::SORT_DESCENDING
	 *
	 * @var array $rules
	 */
	protected $rules;
	
	protected $iterator_index = 0;
	
	public function __construct($id = -1)
	{
		parent::__construct($id);
		$this->_observers = Array();
	}
	
	/**
	 * Add a filtering rule to this object.
	 * 
	 * @param String $field_name A valid field name in the result set.
	 * @param Integer $operation A valid filter type constant: one of DP_Modifier_Filter::FILTER_EQUAL,FILTER_SUBSTRING
	 * @param Mixed $value The filtering value to use.
	 */
	public function addRule( $field_name, $operation, $value )
	{
		$this->rules[] = Array(
			'field' => $field_name,
			'operation' => $operation,
			'value' => $value
		);
	}
	
	/**
	 * Apply rules contained in this modifier to the supplied query.
	 * 
	 * TODO: this kind of violates the abstraction of the modifier from the implementation of the SQL.
	 * @param unknown_type $query
	 */
	public function apply(&$query)
	{
		foreach ($this->rules as $r) {
			if (!isset($r['value']) || $r['value'] == null) {
				continue;
			}
			
			$field = $r['field'];
			$operation = $this->ops_sql[$r['operation']];
			
			$query->where("{$field} {$operation}", $r['value']);
		}
	}
	
	/**
	 * Set the value to match for rules that apply to the given field name.
	 * 
	 * Normally this will be used in conjunction with one rule per field.
	 * 
	 * @param String $name Name of the field
	 * @param String $value Value to change
	 */
	public function setValueForField( $name, $value )
	{
		foreach ($this->rules as &$r) {
			if ($r['field'] == $name) {
				$r['value'] = $value;
			}
		}
	}
	
	/**
	 * Set the value for all rules to the given value.
	 * 
	 * Normally used when there exists only one rule to operate on.
	 * 
	 * @param String $value
	 */
	public function setValue( $value ) {
		foreach ($this->rules as &$r) {
			$r['value'] = $value;
		}
	}
	
	// From Iterator
	
	public function current() {
		return $this->rules[$this->iterator_index];
	}
	
	public function key() {
		return $this->rules[$this->iterator_index];
	}
	
	public function next() {
		$this->iterator_index++;
	}
	
	public function rewind() {
		$this->iterator_index = 0;
	}
	
	public function valid() {
		if (!isset($this->rules[$this->iterator_index])) {
			return true;
		} else {
			return false;
		}
	}

	// From SplSubject
	
	/**
	 * Attach an observer
	 * 
	 * @param SplObserver $observer The observer to attach
	 * @return null
	 */
	public function attach(SplObserver $observer) {
		if (!in_array($observer, $this->_observers)) {
			$this->_observers[] = $observer;
			//$observer->update($this);
			// Observer only updated if something is changed
		}		
	}
	
	/**
	 * Detach an observer
	 * 
	 * @param SplObserver $observer The observer to detach
	 * @return null
	 */
 	public function detach (SplObserver $observer) {
 		if (in_array($observer, $this->_observers)) {
			$observer_key = array_search($this->_observers, $observer);
			$this->_observers[$observer_key] = null;
			
			$reordered_observers = array_values($this->_observers);
			$this->_observers = $reordered_observers;
		}		
 	}
 	
 	/**
 	 * Notify all observers
 	 * 
 	 */
 	public function notify() {
 		foreach($this->_observers as $ob) {
 			$ob->update($this);
 		}
 	}
 	
	// From DP_Originator_Interface
 	
 	/**
	 * Restore internal state from a memento.
	 * 
	 * @param DP_Memento $m State memento.
	 */
	public function setMemento(DP_Memento $m) {
		$state = $m->getState();
		$this->rules = $state['rules'];
	}
	
	/**
	 * Create a memento containing a snapshot of the current internal state.
	 * 
	 * @return DP_Memento current state memento.
	 */
	public function createMemento() {
		$state = Array('rules' => $this->rules);
		return new DP_Memento($state);
	}
	
}