<?php
/**
 * Modifier which represents one or more sorting rules.
 * 
 * Implements iterator so that sorting rules may be iterated. Originator means that
 * the state of the object can be saved and restored.
 * Implements SplSubject so that any change in sorting state will notify the observers.
 */
class DP_Modifier_Sort extends DP_Modifier implements Countable, Iterator, SplSubject, DP_Originator_Interface {
	
	/**
	 * @var Array of observers, implemented as part of SplSubject.
	 */
	protected $_observers;
	
	/**
	 * Array of sorting rules.
	 * 
	 * The array is an associative array of field_name => sorting rule. Where the sorting rule is one of the class
	 * constants:
	 * DP_Modifier_Sort::SORT_ASCENDING or DP_Modifier_Sort::SORT_DESCENDING
	 *
	 * @var array $rules
	 */
	protected $rules;
	
	/**
	 * @var array $sorting_priority Array of field names in the order they should be sorted.
	 */
	protected $sorting_priority;
	
	/**
	 * @var integer $iter_index Index of the iterator
	 */
	protected $iter_index;
	
	/**
	 * @var SORT_ASCENDING Sort a given field in ascending order.
	 */
	const SORT_ASCENDING = 100;
	
	/**
	 * @var SORT_DESCENDING Sort a given field in descending order.
	 */
	const SORT_DESCENDING = 101;
	
	/**
	 * DP_Modifier_Sort Constructor
	 * @param mixed $id Identifier for the DP_Modifier.
	 */
	public function __construct($id = -1) {
		parent::__construct($id);
		
		$this->_observers = Array();
		$this->rules = Array();
		$this->sorting_priority = Array();
		$this->id = $id;
		$this->iter_index = 0;
	}
	
	/**
	 * Sort the specified field in ascending order.
	 * 
	 * @param $field_name Name of the field to sort.
	 */
	function sortAscending($field_name) {
		$this->sort($field_name, DP_Modifier_Sort::SORT_ASCENDING);
	}
	
	/**
	 * Sort the specified field in descending order.
	 * 
	 * @param $field_name Name of the field to sort.
	 */
	function sortDescending($field_name) {
		$this->sort($field_name, DP_Modifier_Sort::SORT_DESCENDING);
	}
	
	/**
	 * Sort the specified field
	 * 
	 * If no direction is specified then the sort method assumes the ascending order.
	 * If sort is called again on the same field then the sort order will be reversed (toggled).
	 * 
	 * @param string $field_name Name of the field to sort.
	 * @param integer $direction Direction to sort, can be DP_Modifier_Sort::SORT_ASCENDING or DP_Modifier_Sort::SORT_DESCENDING
	 */
	function sort($field_name, $direction = null) {
		
		$existing_rule = null;
		
		// Take this field out of the sorting order if it already exists
		for ($i = 0; $i < count($this->sorting_priority); $i++) {
			if ($this->sorting_priority[$i] == $field_name) {
				
				// Store the existing rule but remove it from the priority list.
				$existing_rule = Array($field_name => $this->rules[$field_name]);
				unset($this->sorting_priority[$i]);
				
				break;
			}
		}
		
		// Add the field to the top of the order list
		if (count($this->sorting_priority > 0)) {
			$old_values = array_values($this->sorting_priority);
			
			$this->sorting_priority = array_merge(Array($field_name), $old_values);
		} else {
			$this->sorting_priority[] = $field_name;
		}
		
		// Sort the field descending unless it already exists, then reverse its sorting order.
		if ($direction == null) {
			if (isset($this->rules[$field_name])) {
				// Reverse direction with ternary operator.
				$this->rules[$field_name] = ($this->rules[$field_name] == DP_Modifier_Sort::SORT_DESCENDING) ? DP_Modifier_Sort::SORT_ASCENDING : DP_Modifier_Sort::SORT_DESCENDING;
			} else {
				// Default to ASCENDING.
				$this->rules[$field_name] = DP_Modifier_Sort::SORT_ASCENDING;
			}
		} else {		
			$this->rules[$field_name] = $direction; // Use specified direction.
		}
		
		// Notify observers of sorting rule changes.
		$this->notify();
	}
	
	/**
	 * Get the sort rule for a given field
	 * 
	 * @return DP_Modifier_Sort::SORT_ASCENDING or DP_Modifier_Sort::SORT_DESCENDING or null if not set.
	 */
	public function getSort($field_name) 
	{
		if (in_array($field_name, array_keys($this->rules))) {
			return $this->rules[$field_name];
		} else {
			return null;
		}
	}
	
	/**
	 * Apply rules contained in this modifier to the supplied query.
	 * 
	 * TODO: this kind of violates the abstraction of the modifier from the implementation of the SQL.
	 * @param unknown_type $query
	 */
	public function apply(&$query)
	{
		foreach ($this->sorting_priority as $field) {
			$rule = $this->rules[$field];
			$keyword = '';
			
			if ($rule == DP_Modifier_Sort::SORT_ASCENDING) {
				$keyword = ' ASC';
			}
			
			if ($rule == DP_Modifier_Sort::SORT_DESCENDING) {
				$keyword = ' DESC';
			}
			
			$query->order($field.$keyword);
		}
	}
	
	// From Countable
	
	public function count() {
		return count($this->rules);
	}
	
	// From Iterator
	
	public function current() {
		$sort_field = $this->sorting_priority[$this->iter_index];
		return $this->rules[$sort_field];
	}
	
	public function key() {
		return $this->sorting_priority[$this->iter_index];
	}
	
	public function next() {
		$this->iter_index++;
	}
	
	public function rewind() {
		$this->iter_index = 0;
	}
	
	public function valid() {
		if ($this->iter_index < count($this->sorting_priority)) {
			return true;
		} else {
			return false;
		}
	}

	// From SplSubject
	
	/**
	 * Attach an observer
	 * 
	 * @param SplObserver $observer The observer to attach
	 * @return null
	 */
	public function attach(SplObserver $observer) {
		if (!in_array($observer, $this->_observers)) {
			$this->_observers[] = $observer;
			//$observer->update($this);
			// Observer only updated if something is changed
		}		
	}
	
	/**
	 * Detach an observer
	 * 
	 * @param SplObserver $observer The observer to detach
	 * @return null
	 */
 	public function detach (SplObserver $observer) {
 		if (in_array($observer, $this->_observers)) {
			$observer_key = array_search($this->_observers, $observer);
			$this->_observers[$observer_key] = null;
			
			$reordered_observers = array_values($this->_observers);
			$this->_observers = $reordered_observers;
		}		
 	}
 	
 	/**
 	 * Notify all observers
 	 * 
 	 */
 	public function notify() {
 		foreach($this->_observers as $ob) {
 			$ob->update($this);
 		}
 	}
 	
	// From DP_Originator_Interface
 	
 	/**
	 * Restore internal state from a memento.
	 * 
	 * @param DP_Memento $m State memento.
	 */
	public function setMemento(DP_Memento $m) {
		$state = $m->getState();
		$this->rules = $state['rules'];
		$this->sorting_priority = $state['sorting_priority'];
	}
	
	/**
	 * Create a memento containing a snapshot of the current internal state.
	 * 
	 * @return DP_Memento current state memento.
	 */
	public function createMemento() {
		$state = Array('rules' => $this->rules,
					   'sorting_priority' => $this->sorting_priority);
		return new DP_Memento($state);
	}

}