<?php

/**
 * The base view class, representing the state of a HTML widget.
 * 
 * The DP_View class is the most general representation of a HTML widget. It can be subclassed to produce any 
 * item which should contain the state of a widget. Each DP_View object must have a unique identifier (required by the constructor).
 * 
 * Note: Old DP_View contained visual style information. This DP_View is being re-developed without any visual information because that should
 * be handled by the view script or view helpers. DP_View should be used as a representation of widget state but not necessarily visual state.
 * 
 * Note: The encapsulation of views by other views may not be used in light of inflexibility with the layout. The developer should have full control
 * of the layout regardless of the view state objects.
 * 
 * @package dotproject3
 * @subpackage DP_View
 * @author ebrosnan
 */
class DP_View {
	/**
	 * @var $helper Zend_View_Helper Helper to use when rendering this DP_View object.
	 */
	public $helper = 'dpViewElement';
	
	/**
     * @var Zend_View_Interface Reference to the current view.
     */
    protected $_view;
	
	/**
	 * @var mixed $id This view's identifier.
	 */
	protected $id;

	// TODO: May remove the parent/child/sub view system. It was too inflexible.
	
	/**
	 * @var integer $parent_view_id Identifier of the parent view object.
	 */
	protected $parent_view_id;
	
	/**
	 * @var array $child_views Array of child views inside this view.
	 */
	protected $child_views;
	
	// SubView order
	/**
	 * @var integer APPEND place the child view at the bottom of this DP_View
	 */
	const APPEND = 0;
	/**
	 * @var integer PREPEND place the child view at the top of this DP_View
	 */
	const PREPEND = 1; 
	
	public function __construct($id) {
		$this->id = $id;
		$this->parent_view_id = -1;
		$this->child_views = Array();
	}
	
	/**
	 * Call render when converted to string
	 * 
	 * TODO: Rendering should be performed by a corresponding view helper like Zend_Form
	 * 
	 * @return string HTML output
	 */
	public function __toString() {
		return $this->render();	
	}

	/**
	 * Get the template identifier for this view object.
	 * 
	 * @return string Unique string identifying this view object.
	 */
	public function id() {
		return $this->id;
	}
	
	/**
	 * Get the identifier of the parent view.
	 * 
	 * @return string Unique string identifying the parent view object.
	 */
	public function parentId() {
		return $this->parent_view_id;
	}
	
	/**
	 * Set the identifier of the parent view.
	 * 
	 * @param integer $id Unique string identifying the parent view object.
	 */
	public function setParentViewId($id) {
		$this->parent_view_id = $id;
	}
	
	/**
	 * Add a child view.
	 * 
	 * Add a DP_View which will be inserted inside this view.
	 * @todo Determine standardised behaviour for views that cannot contain other views. Eg. listview
	 */
	public function add(DP_View $view, $location = DP_View::PREPEND) {
		$view->setParentViewId($this->id());
		// $this->child_views[] = $view;
		$this->child_views[] = Array('location'=>$location, 'view'=>$view);
	}
	
	/**
	 * Invoke an associated view helper to render this list
	 * The view helper will output the markup. The only parameter to the helper will be this instance of the object.
	 * 
	 * @param $view Zend_View_Interface The view instance, supplied by the view when calling render() automatically.
	 * @return String HTML Markup
	 */
   	public function render(Zend_View_Interface $view = null)
    {
    	$view = $this->getView();
    	$helper = $this->getHelper();
    	
 		return $view->$helper( $this );
    }
	
	/**
	 * Notify the DP_View Instance that Zend_View will render the template.
	 * 
	 * This allows the DP_View object to make use of view helpers to add javascript inclusions
	 * or modifications to the style sheets as necessary.
	 * 
	 * @param Zend_View $view Instance of the view.
	 */
	public function viewWillRender($view) {
		// Add supporting JavaScript
		// Add supporting CSS
		//Zend_Debug::dump('Calling child notification');
		$this->notifyChildrenWillRender($view);
	}
	
	/**
	 * Notify the child instances of DP_View that Zend_View will render the template.
	 * 
	 * This allows child DP_View objects to add javascript and stylesheets before they are rendered.
	 * 
	 * @param Zend_View $view Instance of the view.
	 */
	protected function notifyChildrenWillRender($view) {
		//Zend_Debug::dump('Notifying '.count($this->child_views).' children of '.$this->Id());
		
		foreach ($this->child_views as $child) {
			//Zend_Debug::dump('notifying children');
			$child['view']->viewWillRender($view);
			//Zend_Debug::dump('Notified child:'.$child['view']->Id());
		}
	}
	
	/**
	 * Render all child DP_View objects into HTML.
	 */
	protected function renderChildren($location = DP_View::PREPEND) {
		$output = "";
		foreach ($this->child_views as $child) {
			if ($child['location'] == $location) {
				//if ($this->getTranslator()) {
				//	$child['view']->setTranslator($this->getTranslator());
				//}
				$output .= $child['view']->render();
			}
		}
		return $output;
	}
	
	/**
	 * Render the child with the given ID string.
	 * 
	 * @param string $id The identifier of the child view.
	 * @return string The HTML output of the child view's render() method.
	 */
	protected function renderChildWithId($id) {
		$output = '';
		foreach ($this->child_views as $child) {
			if ($child->Id() == $id) {
				$output .= $child->render();
			}
		}
		return $output;
	}
	
    /**
     * Get the name of the helper used to render this widget.
     */
    public function getHelper()
    {
    	return $this->helper;
    }
    
    /**
     * Set the name of the helper used to render this widget.
     * @param $helper Zend_View_Helper Helper which transforms this into html.
     */
    public function setHelper($helper)
    {
    	$this->helper = $helper;
    }
    
    /**
     * Set the view interface to use when rendering.
     * 
     * This should be automatically supplied by the view when it attempts to render the object.
     * 
     * @param $view Zend_View_Interface view instance.
     */
    public function setView(Zend_View_Interface $view)
    {
    	$this->_view = $view;
    }
    
    /**
     * Retrieve view object
     *
     * Retrieves from ViewRenderer if none previously set.
     *
     * @return null|Zend_View_Interface
     */
    public function getView()
    {
        if (null === $this->_view) {
            $viewRenderer = Zend_Controller_Action_HelperBroker::getStaticHelper('viewRenderer');
            $this->setView($viewRenderer->view);
        }
        return $this->_view;
    }
}