<?php
/**
 * View that will receive changes in state.
 * 
 * Any view state object that inherits this object will interpret POST or GET variables
 * marked with it's own identifier as being an action that will change its internal state via
 * the method updateStateFromServer(request). This method propagates the request variables to all DP_View_Stateful objects
 * so that they all have a chance in turn to change their state based on those variables. 
 * 
 * Example:
 * 
 * - A widget that changes page next and previous might submit it's own ID as well as a 'next' variable. 
 * - The pager recognises that the variable applies to itself because the ID matches. The pager changes its internal state to represent the current
 * page number. 
 * - The list object observes the change in state and adds a DP_Modifier which will change the page number of the result set.
 * - The result set is generated just before the view renders. The result set is limited by the pager to get the corresponding page.
 * 
 * Old DocBlock:
 * "Views that inherit this class can interpret POST or GET variables
 * and change their internal representation accordingly. Saving the state
 * of the view is also supported through the DP_View_Stateful::saveState() method."
 * 
 * @package dotproject3
 * @subpackage DP_View
 */
class DP_View_Stateful extends DP_View {
	
	/**
	 * @var bool $persist_state Indicates whether this views state should be saved by the session.
	 */
	protected $persist_state;

	/**
	 * @var $_request Zend_Controller_Request Object containing the current request.
	 */
	protected $_request;
	
	function __construct($id) {
		parent::__construct($id);
		
		$this->persist_state = true;
	}
	
	/**
	 * Save the state of the view to a session variable.
	 * 
	 * The state of this object will be saved to a session variable with the same ID as the view object.
	 * 
	 * @todo Allow user to save view state as default, and to load state from default.
	 * @param mixed $state The nominated state variable
	 */
	public function saveState($state) {
		if (true === $this->persist_state) {
			$view_namespace = new Zend_Session_Namespace('DP_View_Stateful');
			
			if (!isset($view_namespace->states)) {
				$view_namespace->states = Array();
			}
			
			$view_namespace->states[$this->id()] = $state;
		}
	}
	
	/**
	 * Load the state of the view from a session variable
	 * 
	 * @todo Allow user to save view state as default, and to load state from default.
	 * @param mixed $default The variable to use if nothing is registered with the session.
	 * @return mixed The state of this view
	 */
	public function loadState($default = null) {
		$view_namespace = new Zend_Session_Namespace('DP_View_Stateful');
		
		if (isset($view_namespace->states) && isset($view_namespace->states[$this->id()])) {
			return $view_namespace->states[$this->id()];
		} else {
			return null;
		}
	}
	
	/**
	 * Set whether the state of this view is persistent i.e should be saved in the session
	 * 
	 * @param bool $persist_state Boolean, if the view's state should be saved in the session.
	 */
	public function setPersistent($persist_state = true) {
		$this->persist_state = $persist_state;
	}
	
	/**
	 * Handle any POST or GET requests.
	 * 
	 * This method tries to access the object's variables in the server request object.
	 * If it finds relevant variables, it will update this object and the changes will be reflected
	 * when the view is rendered. The updates can be saved in the session by calling DP_View_Stateful::saveState()
	 * 
	 * @param mixed $request Server request object.
	 */
	public function updateStateFromServer($request) {
		$this->_request = $request;
	}
}