<?php

/**
 * Class which represents the state of a set of columns.
 * 
 * This class is used to determine the available columns in a list view and also
 * to hold the sorting rules for each of these columns. It is rendered by the view helper DpViewColumns by default.
 * 
 * The columns can also be used by a search widget in order to determine the valid list of columns
 * that can be filtered. In this case, the search widget must know the type of data relevant to the column in order
 * to search.
 * 
 * 
 * @author ebrosnan
 * @package dotproject3
 * @subpackage DP_View
 */
class DP_View_Stateful_Columns extends DP_View_Stateful implements Iterator, Countable
{
	/**
	 * @var $column_definitions Array of column_field_name=>Array containing a number of keys:
	 * title - the name displayed in the output by the view renderer. a human readable version of the column name.
	 * type - to be determined.. this should provide hints on how to search values for this column.
	 */
	protected $column_definitions = Array();
	
	/**
	 * Instance of DP_Modifier_Sort which will be modified by this view when the view receives a relevant sorting parameter.
	 * @var unknown_type
	 */
	protected $sort_modifier = null;
	
	/**
	 * @var Integer internal state of the iterator.
	 */
	protected $iterator_index = 0;
	
	/**
	 * DP_View_Stateful_Columns Constructor
	 * @param string $id Identifier of this columns widget.
	 */
	public function __construct($id)
	{
		parent::__construct($id);
		
		$this->sort_modifier = new DP_Modifier_Sort($id.'-modifier');
		
		// Load state from session.
		$sort_state = $this->loadState();
		
		//Zend_Debug::dump($sort_state);
		if ($sort_state != null && $sort_state instanceof DP_Memento) {
			$this->sort_modifier->setMemento($sort_state);
		}
	}
	
	/**
	 * Set all of the column definitions from a given array.
	 * 
	 * TODO: filter array to force any non legal keys or values to be deleted.
	 * 
	 * @param array $column_definitions Column definition in the format specified 
	 * @see DP_View_Stateful_Columns::column_definitions
	 */
	public function set(Array $column_definitions)
	{
		$this->column_definitions = $column_definitions;
	}
	
	/**
	 * Get an instance of the internal modifier.
	 * 
	 * @return DP_Modifier instance.
	 */
	public function getModifier()
	{
		return $this->sort_modifier;
	}
	
	// From DP_View_Stateful
	
	public function updateStateFromServer($request)
	{
		parent::updateStateFromServer($request);
		
		// Receive state change.
		if ($request->getParam('view_id') == $this->id() && $request->getParam('sort') !== null) {
			$field_to_sort = $request->getParam('sort');
			$this->sort_modifier->sort($field_to_sort);
		}
		
		// Save state
		if (true === $this->persist_state) {
			$this->saveState($this->sort_modifier->createMemento());
		}
	}
	
	// From Countable
	
	public function count() {
		return count($this->column_definitions);
	}
	
	// From Iterator
	
	public function current() {
		$field_names = array_keys($this->column_definitions);
		$field_name = $field_names[$this->iterator_index];
		
		return array_merge($this->column_definitions[$field_name], 
				array('sort'=>$this->sort_modifier->getSort($field_name)));
	}
	
	public function key() {
		$field_names = array_keys($this->column_definitions);
		return $field_names[$this->iterator_index];
	}
	
	public function next() {
		$this->iterator_index++;
	}
	
	public function rewind() {
		$this->iterator_index = 0;
	}
	
	public function valid() {
		$field_names = array_keys($this->column_definitions);
		
		if (isset($field_names[$this->iterator_index])) {
			return true;
		} else {
			return false;
		}
	}
}