<?php

/**
 * A select list that will filter a result set.
 * 
 * The selectfilter is rendered as a select element and maintains an internal modifier which is then applied to the selected
 * list object - giving a filtered list via the selection.
 * 
 * @author ebrosnan
 * @package dotproject3
 * @subpackage DP_View
 */
class DP_View_Stateful_SelectFilter extends DP_View_Stateful implements Iterator
{
	/**
	 * The view helper to use when rendering this object.
	 * @var Zend_View_Helper Helper
	 */
	public $helper = 'dpViewSelectFilter';
	
	/**
	 * @var $modifier DP_Modifier internal modifier instance.
	 */
	protected $modifier = null;
	
	/**
	 * @var $selected_value String the value selected.
	 */
	protected $selected_value = null;
	
	/**
	 * @var $options Array available options in the select list.
	 */
	protected $options = Array();
	
	/**
	 * Current index of the internal iterator
	 * 
	 * Used to support the Iterator SPL pattern.
	 * @var Integer Current index of the internal iterator.
	 */
	protected $iterator_index = 0;
	
	/**
	 * @var $insert_blank_item Boolean whether or not to insert a blank item as the first.
	 * 
	 * The blank item indicates that there is no criteria selected.
	 */
	protected $insert_blank_item = true;
	
	/**
	 * Constructor
	 */
	public function __construct($id = -1, DP_Modifier $mod)
	{
		parent::__construct($id);
		
		$this->modifier = $mod;
		
		$saved_index_memento = $this->loadState();
		if (null !== $saved_index_memento) {
			$this->setSelectedValue($saved_index_memento->getState());
		}
	}
	
	/**
	 * Set options available in this select filter.
	 * 
	 * @param array $options Available options as indexed array containing 2 fields, first will be used as the value, second as the title.
	 */
	public function setOptions(Array $options)
	{
		if (count($options) < 1) {
			throw new DP_Exception('Supplied empty options to SelectFilter. You must supply at least one pair of values.');
		}
		
		$this->options = Array();
		
		foreach ($options as $o) {
			$this->options[] = Array('value' => $o['value'], 'title' => $o['title']);
		}
	}
	
	/**
	 * Get the currently selected value.
	 * 
	 * @return String value selected.
	 */
	public function getSelectedValue()
	{
		return $this->selected_value;
	}
	
	/**
	 * Set the currently selected value.
	 * 
	 * @param String $v Value to select.
	 */
	public function setSelectedValue( $v )
	{
		$this->selected_value = $v;
		$this->modifier->setValue($v); 
	}
	
	/**
	 * Is the item pointed to by the iterator, the one that is selected?
	 * 
	 * @return Boolean true if this item is selected
	 */
	public function isCurrentSelected()
	{
		if ($this->options[$this->iterator_index]['value'] == $this->selected_value) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Set whether a blank item should be inserted in the list or not.
	 * 
	 * @param Boolean $should_insert whether to insert, default is true.
	 */
	public function setInsertBlankItem( $should_insert = true )
	{
		$this->insert_blank_item = (bool)$should_insert;
	}
	
	/**
	 * Does this select filter insert a blank item at the top of the SELECT list?
	 * 
	 * @return Boolean true if blank item will be inserted.
	 */
	public function getInsertBlankItem()
	{
		return $this->insert_blank_item;
	}
	
	/**
	 * Get the internal instance of the modifier.
	 * 
	 * @return DP_Modifier Instance of a modifier.
	 */
	public function getModifier()
	{
		return $this->modifier;
	}
	
	// From DP_View_Stateful
	
	public function updateStateFromServer($request)
	{
		parent::updateStateFromServer($request);
		
		// Receive state change.
		if ($request->getParam('view_id') == $this->id() && $request->getParam('select') !== null) {
			
			//Zend_Debug::dump($this->id().' got update value: '.$request->getParam('select'));
			$value = $request->getParam('select');
			
			$this->setSelectedValue($value);
		}
		
		// Save state
		if (true === $this->persist_state) {
			$this->saveState(new DP_Memento($this->getSelectedValue()));
		}
	}
	
	// From Iterator
	
	public function current() {
		return $this->options[$this->iterator_index];
	}
	
	public function key() {
		return $this->iterator_index;
	}
	
	public function next() {
		$this->iterator_index++;
	}
	
	public function rewind() {
		$this->iterator_index = 0;
	}
	
	public function valid() {
		if (isset($this->options[$this->iterator_index])) {
			return true;
		} else {
			return false;
		}
	}
}