<?php

/**
 * An object which holds the state of a set of tabs used to filter a result set.
 * 
 * Where the tab function is only visual - use dynamic client side tabs.
 * 
 * @author ebrosnan
 * @package dotproject3
 * @subpackage DP_View
 */
class DP_View_Stateful_Tabs extends DP_View_Stateful implements Iterator
{
	/**
	 * The view helper to use when rendering this object.
	 * @var Zend_View_Helper Helper
	 */
	public $helper = 'dpViewTabs';
	
	/**
	 * @var $selected_tab_index Integer Index of the selected tab. Defaults to 0 (First tab)
	 */
	protected $selected_tab_index = 0;
	
	/**
	 * @var Array tab definitions
	 * 	Array([tabIndex] => 
	 * 				Array('title'=>'descriptive title',
	 * 					  'modifier'=>DP_Modifier Instance)
	 *  );
	 */
	protected $tabs;
	
	/**
	 * Index of the internal iterator, used to implement Iterator interface.
	 * 
	 * @var Integer
	 */
	protected $iterator_index = 0;
	
	/**
	 * Constructor for DP_View_Stateful_Tabs
	 * 
	 * @param mixed $id Identifier for the tab widget.
	 */
	public function __construct($id = -1)
	{
		parent::__construct($id);
		
		$this->tabs = Array();
		
		$saved_tab_memento = $this->loadState();
		if (null !== $saved_tab_memento) {
			$this->selected_tab_index = $saved_tab_memento->getState();
		}
	}
	
	/**
	 * Determine if the given index is selected.
	 *
	 * @param $i Integer index of a tab.
	 * @return Boolean true when this index is the selected tab.
	 */
	public function isTabSelected( $i )
	{
		if ($this->selected_tab_index == (int)$i) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Get the currently selected tab index.
	 * 
	 * @return Integer index of the selected tab.
	 */
	public function selectedIndex()
	{
		return $this->selected_tab_index;
	}
	
	/**
	 * Set the currently selected tab index.
	 * 
	 * @param Integer $i Index of the tab to select.
	 */
	public function setSelectedIndex( $i = 0 )
	{
		$i_normal = (int) $i;
		
		$this->selected_tab_index = $i;
	}
	
	/**
	 * Add multiple tabs
	 * 
	 * @todo stub
	 * @param unknown_type $tabs
	 */
	public function addTabs( $tabs )
	{
		
	}
	
	/**
	 * Add a single tab
	 * 
	 * @param array $tab_definition The definition of the tab
	 */
	public function addTab( Array $tab_definition )
	{
		if (!isset($tab_definition['title'])) {
			throw new Exception('Supplied tab definition does not contain a title.');
		}
		
		if (!isset($tab_definition['modifier']) || !$tab_definition['modifier'] instanceof DP_Modifier) {
			throw new Exception('Supplied tab definition does not contain a modifier.');
		}
		
		// Normalise properties.
		$normal_definition = Array(
			'title' => $tab_definition['title'],
			'modifier' => $tab_definition['modifier']
		);
		
		$this->tabs[] = $tab_definition;
	}
	
	/**
	 * Get a single tab definition.
	 * 
	 * @param $tab_index Integer index of tab definition to get.
	 * @return Array|null tab definition or null on out of bounds / no such tab.
	 */
	public function getTab( $tab_index = 0 )
	{
		if (isset($this->tabs[$tab_index])) {
			return $this->tabs[$tab_index];
		} else {
			return null;
		}	
	}
	
	/**
	 * Get the active modifier relative to the tab selected.
	 * 
	 * @return DP_Modifier instance of the modifier to use.
	 */
	public function getActiveModifier()
	{
		return $this->tabs[$this->selectedIndex()]['modifier'];
	}
	
	// From DP_View_Stateful
	
	public function updateStateFromServer($request)
	{
		parent::updateStateFromServer($request);
		
		// Receive state change.
		if ($request->getParam('view_id') == $this->id() && $request->getParam('tab') !== null) {
			$tab_index = $request->getParam('tab');
			$this->setSelectedIndex($tab_index);
		}
		
		// Save state
		if (true === $this->persist_state) {
			$this->saveState(new DP_Memento($this->selected_tab_index));
		}
	}
	
	// From Iterator
	
	public function current() {
		return $this->tabs[$this->iterator_index];
	}
	
	public function key() {
		return $this->iterator_index;
	}
	
	public function next() {
		$this->iterator_index++;
	}
	
	public function rewind() {
		$this->iterator_index = 0;
	}
	
	public function valid() {
		if (isset($this->tabs[$this->iterator_index])) {
			return true;
		} else {
			return false;
		}
	}
}